﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Optimization;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.GoalSeeking {
  public interface IGoalSeekingProblem : IProblem {
    IRegressionProblemData ProblemData { get; set; }

    //rounding ? 
    IEnumerable<double> GetEstimatedGoalValues(IEnumerable<double> parameterValues, bool round = false);
    int Row { get; set; }

    //Models?
    IEnumerable<IRegressionModel> Models { get; }
    void AddModel(IRegressionModel model);
    void RemoveModel(IRegressionModel model);
    event EventHandler ModelsChanged;

    #region targets
    double GetTargetGoal(string target);
    void SetTargetGoal(string target, double goal);

    //variance ?
    double GetTargetWeight(string target);
    void SetTargetWeight(string target, double weight);

    double GetTargetVariance(string target);
    void SetTargetVariance(string target, double variance);

    double GetTargetStepSize(string target);
    void SetTargetStepSize(string target, double stepSize);

    event EventHandler TargetsChanged;
    #endregion

    #region parameters
    ICheckedItemList<StringValue> ControllableParameters { get; }
    DoubleMatrix ControllableParameterBounds { get; }

    IEnumerable<string> GetControllableParameters();
    void SetControllableParameters(IEnumerable<string> parameterNames);

    bool GetParameterActive(string parameter);
    void SetParameterActive(string parameter, bool active);

    double[] GetParameterBounds(string parameterName);
    void SetParameterBounds(string parameterName, double min, double max, double step);
    // set get ParameterStepSize, set get parameters active
    // parameters changed event
    double GetParameterStepSize(string parameter);
    void SetParameterStepSize(string parameter, double stepSize);

    event EventHandler ParametersChanged;
    #endregion
  }
}
